Option Strict On
Option Explicit On

Imports System

Public Class Customer
   Inherits BizObject

   Private mFirstName As String
   Private mLastName As String
   Private mGender As Gender
   Private mMaxTimeToAssign As TimeSpan
   Private mMaxTimeToResolve As TimeSpan


   Public Sub New(ByVal firstName As String, ByVal lastName As String, ByVal gender As Gender)
      mFirstName = firstName
      mLastName = lastName
      mGender = gender
   End Sub

#Region "Editable Properties"
   Public Property FirstName() As String
      Get
         Return mFirstName
      End Get
      Set(ByVal value As String)
         mFirstName = value
      End Set
   End Property

   Public Property LastName() As String
      Get
         Return mLastName
      End Get
      Set(ByVal value As String)
         mLastName = value
      End Set
   End Property

   Public Property Gender() As Gender
      Get
         Return mGender
      End Get
      Set(ByVal value As Gender)
         mGender = value
      End Set
   End Property

   Public Property MaxTimeToAssign() As TimeSpan
      Get
         Return mMaxTimeToAssign
      End Get
      Set(ByVal value As TimeSpan)
         mMaxTimeToAssign = value
      End Set
   End Property

   Public Property MaxTimeToResolve() As TimeSpan
      Get
         Return mMaxTimeToResolve
      End Get
      Set(ByVal value As TimeSpan)
         mMaxTimeToResolve = value
      End Set
   End Property

#End Region

#Region "BizObject Overrides"

   Public Overrides Sub Save()
      ' Do save here. raise exception on error
   End Sub

   Public Overrides ReadOnly Property CanRead( _
            ByVal propertyName As String) _
            As Boolean
      Get
         Select Case propertyName
            Case "Gender"
               Dim principal As Security.Principal.IPrincipal = Threading.Thread.CurrentPrincipal
               Return principal.IsInRole("Manager")
            Case Else
               Return True
         End Select
      End Get
   End Property

   Public Overrides ReadOnly Property CanWrite(ByVal propertyName As String) As Boolean
      Get
         Select Case propertyName
            Case "Gender"
               Dim principal As Security.Principal.IPrincipal = Threading.Thread.CurrentPrincipal
               Return principal.IsInRole("Manager")
            Case "LastName"
               Dim principal As Security.Principal.IPrincipal = Threading.Thread.CurrentPrincipal
               Return principal.IsInRole("Manager")
            Case Else
               Return True
         End Select
      End Get
   End Property
#End Region


#Region "IDataErrorInfo Implementation"

   Default Public Overrides ReadOnly Property Item( _
            ByVal columnName As String) _
            As String 
      Get
         Select Case columnName
            Case "FirstName"
               If String.IsNullOrEmpty(Me.FirstName) Then
                  Return "First name can't be empty"
               End If
            Case "LastName"
               If String.IsNullOrEmpty(Me.LastName) Then
                  Return "Last name can't be empty"
               End If
         End Select
         Return String.Empty
      End Get
   End Property
#End Region

End Class
