Option Strict On
Option Explicit On

Imports System

Public Enum IncidentStatus
   Unknown = 0
   Submitted
   InProcess
   Resolved
End Enum

Public Enum IncidentSeriousness
   Unknown = 0
   StillWorks
   WorksBadly
   NotWorking
End Enum

Public Enum IncidentState
   Unknown = 0
   Ok
   AssignmentDue
   AssignmentLate
   ResolvedDue
   ResolvedLate
   ResolvedDueWithoutAssignment
End Enum

Public Class SupportIncident
   Inherits BizObject

   Private mTitle As String
   Private mDescription As String
   Private mSubmittedBy As Customer
   Private mSubmittedDate As DateTime
   Private mDueDate As DateTime
   Private mStatus As IncidentStatus
   Private mSeriousness As IncidentSeriousness
   Private mAssignedTo As Employee

   Public Sub New( _
            ByVal title As String, _
            ByVal description As String, _
            ByVal submittedBy As Customer, _
            ByVal submittedDate As DateTime, _
            ByVal dueDate As DateTime, _
            ByVal status As IncidentStatus, _
            ByVal seriousness As IncidentSeriousness, _
            ByVal assignedTo As Employee)
      Me.mTitle = title
      Me.mDescription = description
      Me.mSubmittedBy = submittedBy
      Me.mSubmittedDate = submittedDate
      Me.mDueDate = dueDate
      Me.mStatus = status
      Me.mAssignedTo = assignedTo
   End Sub

#Region "Editable Properties"

   Public Property Title() As String
      Get
         Return mTitle
      End Get
      Set(ByVal value As String)
         mTitle = value
      End Set
   End Property

   Public Property Description() As String
      Get
         Return mDescription
      End Get
      Set(ByVal value As String)
         mDescription = value
      End Set
   End Property

   Public Property SubmittedBy() As Customer
      Get
         Return mSubmittedBy
      End Get
      Set(ByVal value As Customer)
         mSubmittedBy = value
      End Set
   End Property

   Public Property SubmittedDate() As DateTime
      Get
         Return mSubmittedDate
      End Get
      Set(ByVal value As DateTime)
         mSubmittedDate = value
      End Set
   End Property

   Public Property DueDate() As DateTime
      Get
         Return mDueDate
      End Get
      Set(ByVal value As DateTime)
         mDueDate = value
      End Set
   End Property

   Public Property Status() As IncidentStatus
      Get
         Return mStatus
      End Get
      Set(ByVal value As IncidentStatus)
         mStatus = value
      End Set
   End Property

   Public Property Seriousness() As IncidentSeriousness
      Get
         Return mSeriousness
      End Get
      Set(ByVal value As IncidentSeriousness)
         mSeriousness = value
      End Set
   End Property

   Public Property AssignedTo() As Employee
      Get
         Return mAssignedTo
      End Get
      Set(ByVal value As Employee)
         mAssignedTo = value
      End Set
   End Property
#End Region

#Region "Calcualted Properties"
   Public ReadOnly Property IncidentState() As IncidentState
      Get
         If mStatus <> IncidentStatus.Resolved Then
            Dim resolvedDelay As TimeSpan = DateTime.Now - mDueDate
            If (mSubmittedBy IsNot Nothing AndAlso resolvedDelay > mSubmittedBy.MaxTimeToAssign) Or _
                        resolvedDelay > ApplicationData.MaxTimeToAssign Then
               Return IncidentState.ResolvedLate
            End If
            Dim resolvedDue As TimeSpan = DateTime.Now - mDueDate - ApplicationData.CurrentUser.ResolvedDueBuffer
            If (mSubmittedBy IsNot Nothing AndAlso resolvedDue > mSubmittedBy.MaxTimeToAssign) Or _
                        resolvedDue > ApplicationData.MaxTimeToAssign Then
               If (mAssignedTo Is Nothing) Then
                  Return BObjects.IncidentState.ResolvedDueWithoutAssignment
               End If
               Return IncidentState.ResolvedDue
            End If
         End If
         If mAssignedTo Is Nothing Then
            Dim submittedDelay As TimeSpan = DateTime.Now - mSubmittedDate
            If (mSubmittedBy IsNot Nothing AndAlso submittedDelay > mSubmittedBy.MaxTimeToAssign) Or _
                        submittedDelay > ApplicationData.MaxTimeToAssign Then
               Return IncidentState.AssignmentLate
            End If
            Dim submittedDue As TimeSpan = DateTime.Now - mSubmittedDate - ApplicationData.CurrentUser.AssignedDueBuffer
            If (mSubmittedBy IsNot Nothing AndAlso submittedDue > mSubmittedBy.MaxTimeToAssign) Or _
                        submittedDue > ApplicationData.MaxTimeToAssign Then
               Return IncidentState.AssignmentDue
            End If
         End If
         Return BObjects.IncidentState.Ok
      End Get
   End Property
#End Region

#Region "BizObject Overrides"

   Public Overrides Sub Save()
      ' Do save here
   End Sub
#End Region

End Class
