Option Strict On
Option Explicit On

Imports System.ComponentModel
Imports System

Public Class AuthorizationDecorator
   Inherits BindingSupportDecoratorBase

   Public Sub New()
      AddHandler Me.DataContextChanged, AddressOf Decorator_DataContextChanged
      AddHandler Me.Loaded, AddressOf Decorator_Loaded
   End Sub

   Private Sub Decorator_Loaded( _
            ByVal sender As Object, _
            ByVal e As Windows.RoutedEventArgs)
      Refresh()
      Authorize()
   End Sub

   Private Sub Decorator_DataContextChanged( _
            ByVal sender As Object, _
            ByVal e As Windows.DependencyPropertyChangedEventArgs)
      Refresh()
   End Sub

   Protected Sub AuthorizeElement( _
            ByVal element As Windows.FrameworkElement, _
            ByVal binding As Windows.Data.Binding, _
            ByVal prop As Windows.DependencyProperty, _
            ByVal associatedLabel As Windows.Controls.Label)
      Dim expression As Windows.Data.BindingExpression = element.GetBindingExpression(prop)
      Dim bizObject As BObjects.BizObject = TryCast(Me.BoundObject, BObjects.BizObject)
      If expression IsNot Nothing AndAlso bizObject IsNot Nothing Then
         Dim canWrite As Boolean = bizObject.CanWrite(binding.Path.Path)
         Dim propertyInfo As Reflection.PropertyInfo = element.GetType().GetProperty("IsReadOnly", Reflection.BindingFlags.FlattenHierarchy Or Reflection.BindingFlags.Instance Or Reflection.BindingFlags.Public)
         If Not propertyInfo Is Nothing Then
            propertyInfo.SetValue(element, (Not canWrite), New Object() {})
         Else
            element.IsEnabled = canWrite
         End If

         Dim canRead As Boolean = bizObject.CanRead(binding.Path.Path)
         If canRead Then
            element.Visibility = Visibility.Visible
            If associatedLabel IsNot Nothing Then
               associatedLabel.Visibility = Visibility.Visible
            End If
         Else
            element.Visibility = Visibility.Collapsed
            If associatedLabel IsNot Nothing Then
               associatedLabel.Visibility = Visibility.Collapsed
            End If
         End If
      End If
   End Sub

   Public Sub Authorize()
      Dim dataErrorInfo As IDataErrorInfo = TryCast(Me.BoundObject, IDataErrorInfo)
      If dataErrorInfo IsNot Nothing Then
         Me.ForAll(AddressOf AuthorizeElement)
      End If
   End Sub

End Class
