Option Strict On
Option Explicit On

Imports System.ComponentModel
Imports System

Public Class ValidationDecorator
   Inherits Decorator

   Private Delegate Sub FoundBindingCallbackDelegate(ByVal element As FrameworkElement, _
               ByVal binding As Binding, ByVal dp As DependencyProperty)
   Private _firstInvalidElement As FrameworkElement
   Private _backupStyles As Dictionary(Of DependencyObject, Style) = New Dictionary(Of DependencyObject, Style)()

   Public Sub New()
      'this.DataContextChanged += new DependencyPropertyChangedEventHandler(ErrorProvider_DataContextChanged);
      'this.Loaded += new RoutedEventHandler(ErrorProvider_Loaded);
      ' If you wish to validate on load
      AddHandler Me.DataContextChanged, AddressOf Decorator_DataContextChanged
      AddHandler Me.Loaded, AddressOf Decorator_Loaded
   End Sub

   Private Sub Decorator_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
      Validate()
   End Sub

   Private Sub Decorator_DataContextChanged( _
            ByVal sender As Object, _
            ByVal e As DependencyPropertyChangedEventArgs)
      If e.OldValue IsNot Nothing Then
         Dim oldNotify As INotifyPropertyChanged = TryCast(e.OldValue, INotifyPropertyChanged)
         RemoveHandler oldNotify.propertychanged, AddressOf UIElement_PropertyChanged
      End If
      If e.NewValue IsNot Nothing Then
         Dim newNotify As INotifyPropertyChanged = TryCast(e.OldValue, INotifyPropertyChanged)
         AddHandler newNotify.propertychanged, AddressOf UIElement_PropertyChanged
      End If
      Validate()
   End Sub

   Public Function Validate() As bool
      Dim isValid As Boolean = True
            _firstInvalidElement = null;
      Dim dataErrorProvider As IDataErrorInfo = TryCast(Me.DataContext, IDataErrorInfo)
      If dataErrorProvider IsNot Nothing Then
         Dim bindings As List(Of Binding) = ClearInternal()

         For Each binding As Binding In bindings
            Dim message As String = dataErrorProvider(binding.Path.Path)
            If String.IsNullOrEmpty(message) Then
               isValid = False
               '  Display the error on any elements bound to the property
               DisplayErrors(Me.Parent)
         Next
      End If
      Return isValid
   End Function

   Private Sub DisplayErrors(ByVal parent As FrameworkElement, ByVal binding As Binding)
      FindBindingsRecursively(Me.Parent, AddressOf DisplayError)
   End Sub

   Private Sub DisplayError( _
            ByVal parent As FrameworkElement, _
            ByVal binding As Binding, _
            ByVal testElement As FrameworkElement, _
            ByVal testBinding As Binding, _
            ByVal prop As DependencyProperty)
      If binding.Path.Path = testBinding.Path.Path Then
         Dim expression As BindingExpression = element.GetBindingExpression(prop)
         Dim err As New ValidationError(New ExceptionValidationRule(), expression, errorMessage, null)
         System.Windows.Controls.Validation.MarkInvalid(expression, err)
         If _firstInvalidElement Is Nothing Then
            _firstInvalidElement = testElement
         End If
      End If
   End Sub

   Public ReadOnly Property FirstInvalidElement() As FrameworkElement
      Get
         Return _firstInvalidElement
      End Get
   End Property

   Public Sub Clear()
      ClearInternal()
   End Sub

   Private Function ClearInternal() As List(Of Binding)
      Dim bindings As New List(Of Binding)
      FindBindingsRecursively(Me.Parent, AddressOf ClearBinding)
                    delegate(FrameworkElement element, Binding binding, DependencyProperty dp)
                    {
                        // Remember this bound element. We'll use this to display error messages for each property.
                        bindings.Add(binding);
                    });
            return bindings;
   End Function

   Private Sub ClearBinding( _
            ByVal bindings As List(Of Binding), _
            ByVal element As FrameworkElement, _
            ByVal binding As Binding, _
            ByVal dp As DependencyProperty)
      bindings.Add(binding)
   End Sub

   Private Sub UIElement_PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
      If e.PropertyName <> "IsValid" Then
         Validate()
      End If
   End Sub


End Class
