Option Strict On
Option Explicit On

Imports System.ComponentModel
Imports System

Public Class ValidationDecorator
   Inherits BindingSupportDecoratorBase

   Private mInvalidElement As Windows.FrameworkElement
   Private mIsValid As Boolean

   Public Sub New()
      AddHandler Me.DataContextChanged, AddressOf Decorator_DataContextChanged
      AddHandler Me.Loaded, AddressOf Decorator_Loaded
   End Sub

   Private Sub Decorator_Loaded( _
            ByVal sender As Object, _
            ByVal e As Windows.RoutedEventArgs)
      Refresh()
      Validate()
   End Sub

   Private Sub Decorator_DataContextChanged( _
            ByVal sender As Object, _
            ByVal e As Windows.DependencyPropertyChangedEventArgs)
      Refresh()
   End Sub

   Public Overrides Sub Refresh()
      MyBase.Refresh()
      Me.ForAll(AddressOf HookElementEvents)
   End Sub

   Private Sub HookElementEvents( _
            ByVal frameworkElement As Windows.FrameworkElement, _
            ByVal binding As Windows.Data.Binding, _
            ByVal prop As Windows.DependencyProperty, _
            ByVal associatedLabel As Windows.Controls.Label)
      AddHandler frameworkElement.LostFocus, AddressOf ElementLostFocus
   End Sub

   Private Sub ElementLostFocus(ByVal sender As Object, ByVal e As Windows.RoutedEventArgs)
      Validate()
   End Sub

   Protected Sub ValidateElement( _
            ByVal element As Windows.FrameworkElement, _
            ByVal binding As Windows.Data.Binding, _
            ByVal prop As Windows.DependencyProperty, _
            ByVal associatedLabel As Windows.Controls.Label)
      Dim expression As Windows.Data.BindingExpression = element.GetBindingExpression(prop)
      Dim dataErrorInfo As IDataErrorInfo = TryCast(Me.BoundObject, IDataErrorInfo)
      If expression IsNot Nothing AndAlso dataErrorInfo IsNot Nothing Then
         Dim message As String = dataErrorInfo(binding.Path.Path)
         If String.IsNullOrEmpty(message) Then
            Windows.Controls.Validation.ClearInvalid(expression)
         Else
            Dim validationError As Windows.Controls.ValidationError = New Windows.Controls.ValidationError(New Windows.Controls.ExceptionValidationRule(), expression, message, Nothing)
            Windows.Controls.Validation.MarkInvalid(expression, validationError)
            Me.mIsValid = False
            If Me.mInvalidElement Is Nothing Then
               Me.mInvalidElement = element
            End If
         End If
      End If
   End Sub

   Public Function Validate() As Boolean
      Me.mIsValid = True
      Me.mInvalidElement = Nothing
      Dim dataErrorInfo As IDataErrorInfo = TryCast(Me.BoundObject, IDataErrorInfo)
      If dataErrorInfo IsNot Nothing Then
         Me.ForAll(AddressOf ValidateElement)
      End If
      Return Me.mIsValid
   End Function

   Public ReadOnly Property FirstInvalidElement() As Windows.FrameworkElement
      Get
         Return Me.mInvalidElement
      End Get
   End Property

End Class
