''' <summary>
''' This class acts as a base class for Row Objects and Call Objects.
''' </summary>
''' <remarks>
''' <para>Most call objects and row objects derive-- usually indrectly-- from 
''' this class.</para>
''' </remarks>
Public MustInherit Class DbObject

    ''' <summary>
    ''' Returns the value of the specified property or field, optionally converting to
    ''' <paramref name="AsType" /> type.
    ''' </summary>
    Public Overridable Function GetProperty(ByVal Name As String, ByVal AsType As Type) As Object
        Dim Reflect As ReflectionCache
        Dim Member As MemberInfo
        Dim Value As Object = Nothing

        Reflect = ReflectionCache.CreateInstance(Me.GetType)
        Member = Reflect.GetMember(Name)
        If TypeOf Member Is FieldInfo Then
            Value = CType(Member, FieldInfo).GetValue(Me)
        ElseIf TypeOf Member Is PropertyInfo Then
            Value = CType(Member, PropertyInfo).GetValue(Me, Nothing)
        End If
        If AsType Is Nothing OrElse Value.GetType Is AsType Then
            Return Value
        Else
            Return Convert.ChangeType(Value, AsType)
        End If

    End Function

    ''' <summary>
    ''' Sets the value of the specified property or field.
    ''' </summary>
    Public Overridable Sub SetProperty(ByVal Name As String, ByVal Value As Object)
        Dim Reflect As ReflectionCache
        Dim Member As MemberInfo
        Dim ActualValue As Object

        Reflect = ReflectionCache.CreateInstance(Me.GetType)
        Member = Reflect.GetMember(Name)
        If TypeOf Member Is FieldInfo Then
            If CType(Member, FieldInfo).FieldType.IsAssignableFrom(Value.GetType) Then
                ActualValue = Value
            Else
                ActualValue = Convert.ChangeType(Value, CType(Member, FieldInfo).FieldType)
            End If
            CType(Member, FieldInfo).SetValue(Me, ActualValue)
        ElseIf TypeOf Member Is PropertyInfo Then
            If CType(Member, PropertyInfo).PropertyType Is Value.GetType Then
                ActualValue = Value
            Else
                ActualValue = Convert.ChangeType(Value, CType(Member, PropertyInfo).PropertyType)
            End If
            CType(Member, PropertyInfo).SetValue(Me, ActualValue, Nothing)
        End If

    End Sub

End Class

