''' <summary>
''' This interface represents a data store: SQL Server, MSMQ, WS, etc.
''' </summary>
Public Interface IDataService

    ''' <summary>
    ''' Rolls back a transaction started by a call to <see cref="BeginTransaction" />.
    ''' </summary>
    ''' <exception cref="InvalidOperationException">The implementation should throw
    ''' this exception if there is no transaction currently in progress.</exception>
    Sub Abort()

    ''' <summary>
    ''' Starts a new transaction. If a transaction is already in progress, the data 
    ''' service should not throw an exception.
    ''' </summary>
    Sub BeginTransaction()

    ''' <summary>
    ''' Commits a transaction previously started by a call to 
    ''' <see cref="BeginTransaction"/>.
    ''' </summary>
    ''' <exception cref="InvalidOperationException">The implementation should throw this
    ''' exception if there is no transaction currently in progress.</exception>
    Sub Commit()

    ''' <summary>
    ''' Gets / Sets the name of data source's connection string.
    ''' </summary>
    ''' <remarks>The implementation should enforce write-once semantics for this property.
    ''' Once set it cannot be changed.</remarks>
    ''' <exception cref="InvalidOperationException">The implementation should throw this
    ''' exception if a second attempt is made to set the property.</exception>
    Property ConnectionName() As String

    ''' <summary>
    ''' Executes the specified call object against the underlying data store.
    ''' </summary>
    Sub Execute(ByVal Command As CallObject)

    ''' <summary>
    ''' Executes the specified query, returning the results as a collection of 
    ''' <see cref="RowObject"/> instances.
    ''' </summary>
    Function Query(Of T As RowObject)(ByVal Command As CallObject) As List(Of T)

End Interface
