CREATE SCHEMA STDA AUTHORIZATION dbo
GO

IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_SCHEMA = 'STDA' AND ROUTINE_NAME = 'CustomersFetch')
BEGIN
	DROP PROCEDURE STDA.CustomersFetch
END
GO
CREATE PROCEDURE STDA.CustomersFetch
	@CustomerID NCHAR(5),
	@CompanyName NVARCHAR(40) OUTPUT,
	@ContactName NVARCHAR(30) OUTPUT,
	@ContactTitle NVARCHAR(30) OUTPUT,
	@Address NVARCHAR(60) OUTPUT,
	@City NVARCHAR(15) OUTPUT,
	@Region NVARCHAR(15) OUTPUT,
	@PostalCode NVARCHAR(10) OUTPUT,
	@Country NVARCHAR(15) OUTPUT,
	@Phone NVARCHAR(24) OUTPUT,
	@Fax NVARCHAR(24) OUTPUT
AS
	SELECT
		@CompanyName = CompanyName,
		@ContactName = ContactName,
		@ContactTitle = ContactTitle,
		@Address = Address,
		@City = City,
		@Region = Region,
		@PostalCode = PostalCode,
		@Country = Country,
		@Phone = Phone,
		@Fax = Fax
	FROM Customers
	WHERE CustomerID = @CustomerID
	
	RETURN @@ROWCOUNT
GO

IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_SCHEMA = 'STDA' AND ROUTINE_NAME = 'CustomersOverviewFetch')
BEGIN
	DROP PROCEDURE STDA.CustomersOverviewFetch
END
GO
CREATE PROCEDURE STDA.CustomersOverviewFetch
AS
	SELECT 
		CustomerID, CompanyName, 
		City, Region, 
		Country 
	FROM Customers
	
GO

IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_SCHEMA = 'STDA' AND ROUTINE_NAME = 'CustomersUpdate')
BEGIN
	DROP PROCEDURE STDA.CustomersUpdate
END
GO
CREATE PROCEDURE STDA.CustomersUpdate
	@CustomerID NCHAR(5),
	@CompanyName NVARCHAR(40),
	@ContactName NVARCHAR(30),
	@ContactTitle NVARCHAR(30),
	@Address NVARCHAR(60),
	@City NVARCHAR(15),
	@Region NVARCHAR(15),
	@PostalCode NVARCHAR(10),
	@Country NVARCHAR(15),
	@Phone NVARCHAR(24),
	@Fax NVARCHAR(24)
AS
	IF EXISTS (SELECT * FROM Customers WHERE CustomerID = @CustomerID)
	BEGIN
		UPDATE Customers
		SET
			CompanyName = @CompanyName,
			ContactName = @ContactName,
			ContactTitle = @ContactTitle,
			Address = @Address,
			City = @City,
			Region = @Region,
			PostalCode = @PostalCode,
			Country = @Country,
			Phone = @Phone,
			Fax = @Fax
		WHERE CustomerID = @CustomerID	
	END ELSE BEGIN
		INSERT INTO Customers
		(
			CustomerID, CompanyName, ContactTitle,
			Address, City, Region,
			PostalCode, Country, Phone,
			Fax
		) VALUES (
			@CustomerID, @CompanyName, @ContactTitle,
			@Address, @City, @Region,
			@PostalCode, @Country, @Phone,
			@Fax
		)	
	END 	
	RETURN @@ROWCOUNT

GO
